@@ -56,7 +56,7 @@
 static char const GCC_UNUSED copyright[] =
     "@(#)Copyright (c) 2004 -- Ohio University.\n";
 static char const GCC_UNUSED rcsid[] =
-    "$Header: /usr/local/cvs/tcptrace/compress.c,v 5.8 2003/11/19 14:38:01 sdo Exp $";
+    "$Header: /usr/local/cvs/tcptrace/compress.c,v 5.9 2004/10/01 21:42:34 mramadas Exp $";
 
 
 #include "compress.h"
@@ -626,7 +626,7 @@ PipeFitting(
     FILE *f_header,
     FILE *f_orig_stdin)
 {
-    char buf[4096];		/* just a big buffer */
+    char buf[COMP_HDR_SIZE];		/* just a big buffer */
     int len;
 
     // Fix the file synchronization problems and undefined behavior exhibited
