@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *               2002, 2003, 2004
  *	Ohio University.  All rights reserved.
  *
  * ---
@@ -51,8 +52,8 @@
  *		ostermann@cs.ohiou.edu
  *		http://www.tcptrace.org/
  */
-static char const rcsid_slice[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/mod_slice.h,v 1.2 2001/05/31 20:55:02 sdo Exp $";
+static char const GCC_UNUSED rcsid_slice[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/mod_slice.h,v 1.3 2003/11/19 14:38:07 sdo Exp $";
 
 
 /* header file for slice.c */
