@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *               2002, 2003, 2004
  *	Ohio University.
  *
  * ---
@@ -50,15 +51,15 @@
  * 		Athens, OH
  *		http://www.tcptrace.org/
  */
-static char const copyright[] =
-    "@(#)Copyright (c) 2001 -- Ohio University.\n";
-static char const rcsid[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/poolaccess.c,v 5.3 2001/05/31 21:26:43 sdo Exp $";
-
-
 #include "tcptrace.h"
+static char const GCC_UNUSED copyright[] =
+    "@(#)Copyright (c) 2004 -- Ohio University.\n";
+static char const GCC_UNUSED rcsid[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/poolaccess.c,v 5.5 2003/11/19 14:38:04 sdo Exp $";
+
 
 static long tcp_pair_pool = -1;
+static long udp_pair_pool = -1;
 static long seqspace_pool = -1;
 static long ptp_snap_pool = -1;
 static long ptp_ptr_pool  = -1;
@@ -86,6 +87,27 @@ FreeTcpPair(
   PoolFree(tcp_pair_pool, ptr);
 }
 
+udp_pair *
+MakeUdpPair(
+	    void)
+{
+  udp_pair	*ptr = NULL;
+
+  if (udp_pair_pool < 0) {
+    udp_pair_pool = MakeMemPool(sizeof(udp_pair), 0);
+  }
+  
+  ptr = PoolMalloc(udp_pair_pool, sizeof(udp_pair));
+  return ptr;
+}
+
+void
+FreeUdpPair(
+	    udp_pair *ptr)
+{
+  PoolFree(udp_pair_pool, ptr);
+}
+
 seqspace *
 MakeSeqspace(
 	     void)
