@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+ *               2002, 2003, 2004
  *	Ohio University.
  *
  * ---
@@ -51,10 +52,11 @@
  *		ostermann@cs.ohiou.edu
  *		http://www.tcptrace.org/
  */
-static char const copyright[] =
-    "@(#)Copyright (c) 2001 -- Ohio University.\n";
-static char const rcsid[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/rexmit.c,v 5.10 2001/08/30 21:56:26 mramadas Exp $";
+#include "tcptrace.h"
+static char const GCC_UNUSED copyright[] =
+    "@(#)Copyright (c) 2004 -- Ohio University.\n";
+static char const GCC_UNUSED rcsid[] =
+    "@(#)$Header: /usr/local/cvs/tcptrace/rexmit.c,v 5.14 2003/11/19 14:38:05 sdo Exp $";
 
 
 /* 
@@ -78,22 +80,37 @@ It can also keep track of packets that come out of order.
 */
 
 
-#include "tcptrace.h"
 
 /* locally global variables*/
 
 
 /* local routine definitions*/
-static void insert_seg_between(quadrant *, segment *, segment *, segment *);
-static void collapse_quad(quadrant *);
-static segment *create_seg(seqnum, seglen);
-static quadrant *whichquad(seqspace *, seqnum);
-static quadrant *create_quadrant(void);
-static int addseg(tcb *, quadrant *, seqnum, seglen, Bool *);
-static void rtt_retrans(tcb *, segment *);
-static enum t_ack rtt_ackin(tcb *, segment *, Bool rexmit);
-static void dump_rtt_sample(tcb *, segment *, double);
-static void graph_rtt_sample(tcb *, segment *, unsigned long);
+static void insert_seg_between (quadrant *,
+				segment *,
+				segment *,
+				segment *);
+static void collapse_quad (quadrant *);
+static segment *create_seg (seqnum,
+			    seglen);
+static quadrant *whichquad (seqspace *,
+			    seqnum);
+static quadrant *create_quadrant (void);
+static int addseg (tcb *,
+		   quadrant *,
+		   seqnum,
+		   seglen,
+		   Bool *);
+static void rtt_retrans (tcb *,
+			 segment *);
+static enum t_ack rtt_ackin (tcb *,
+			     segment *,
+			     Bool rexmit);
+static void dump_rtt_sample (tcb *,
+			     segment *,
+			     double);
+static void graph_rtt_sample (tcb *,
+			      segment *,
+			      unsigned long);
 
 
 
@@ -102,11 +119,11 @@ static void graph_rtt_sample(tcb *, segment *, unsigned long);
  *   returns: number of retransmitted bytes in segment, 0 if not a rexmit
  *            *pout_order to to TRUE if segment is out of order
  */
-int rexmit(
-    tcb *ptcb,
-    seqnum seq,
-    seglen len,
-    Bool  *pout_order)
+int
+rexmit (tcb * ptcb,
+	seqnum seq,
+	seglen len,
+	Bool * pout_order)
 {
     seqspace *sspace = ptcb->ss;
     seqnum seq_last = seq + len - 1;
@@ -117,39 +134,38 @@ int rexmit(
     *pout_order = FALSE;
 
     /* see which quadrant it starts in */
-    pquad = whichquad(sspace,seq);
+    pquad = whichquad (sspace, seq);
 
     /* add the new segment into the segment database */
-    if (BOUNDARY(seq,seq_last)) {
+    if (BOUNDARY (seq, seq_last)) {
 	/* lives in two different quadrants (can't be > 2) */
-	seqnum seq1,seq2;
-	u_long len1,len2;
+	seqnum seq1, seq2;
+	u_long len1, len2;
 
 	/* in first quadrant */
 	seq1 = seq;
-	len1 = LAST_SEQ(QUADNUM(seq1)) - seq1 + 1;
-	rexlen = addseg(ptcb,pquad,seq1,len1,pout_order);
+	len1 = LAST_SEQ (QUADNUM (seq1)) - seq1 + 1;
+	rexlen = addseg (ptcb, pquad, seq1, len1, pout_order);
 
 	/* in second quadrant */
-	seq2 = FIRST_SEQ(QUADNUM(seq_last));
+	seq2 = FIRST_SEQ (QUADNUM (seq_last));
 	len2 = len - len1;
-	rexlen += addseg(ptcb,pquad->next,seq2,len2,pout_order);
+	rexlen += addseg (ptcb, pquad->next, seq2, len2, pout_order);
     } else {
-	rexlen = addseg(ptcb,pquad,seq,len,pout_order);
+	rexlen = addseg (ptcb, pquad, seq, len, pout_order);
     }
 
-    return(rexlen);
+    return (rexlen);
 }
 
 
 /********************************************************************/
 static int
-addseg(
-    tcb *ptcb,
-    quadrant *pquad,
-    seqnum thisseg_firstbyte,
-    seglen len,
-    Bool  *pout_order)
+addseg (tcb * ptcb,
+	quadrant * pquad,
+	seqnum thisseg_firstbyte,
+	seglen len,
+	Bool * pout_order)
 {
     seqnum thisseg_lastbyte = thisseg_firstbyte + len - 1;
     segment *pseg;
@@ -161,7 +177,7 @@ addseg(
     pseg = pquad->seglist_head;
 
     /* (optimize expected case, it just goes at the end) */
-    if (pquad->seglist_tail && 
+    if (pquad->seglist_tail &&
 	(thisseg_firstbyte > pquad->seglist_tail->seq_lastbyte))
 	pseg = NULL;
     for (; pseg != NULL; pseg = pseg->next) {
@@ -180,13 +196,13 @@ addseg(
 		*pout_order = TRUE;
 
 	    /* make a new segment record for it */
-	    pseg_new = create_seg(thisseg_firstbyte, len);
-	    insert_seg_between(pquad,pseg_new,pseg->prev,pseg);
+	    pseg_new = create_seg (thisseg_firstbyte, len);
+	    insert_seg_between (pquad, pseg_new, pseg->prev, pseg);
 
 	    /* see if we overlap the next segment in the list */
 	    if (thisseg_lastbyte < pseg->seq_firstbyte) {
 		/* we don't overlap, so we're done */
-		return(rexlen);
+		return (rexlen);
 	    } else {
 		/* overlap him, split myself in 2 */
 
@@ -207,12 +223,12 @@ addseg(
 	    /* starts within this recorded sequence */
 	    ++pseg->retrans;
 	    if (!split)
-		rtt_retrans(ptcb,pseg);  /* must be a retransmission */
+		rtt_retrans (ptcb, pseg);	/* must be a retransmission */
 
 	    if (thisseg_lastbyte <= pseg->seq_lastbyte) {
 		/* entirely contained within this sequence */
 		rexlen += len;
-		return(rexlen);
+		return (rexlen);
 	    }
 	    /* else */
 	    /* we extend beyond this sequence, split ourself in 2 */
@@ -227,84 +243,82 @@ addseg(
 
     /* if we got to the end, then it doesn't go BEFORE anybody, */
     /* tack it onto the end */
-    pseg_new = create_seg(thisseg_firstbyte, len);
-    insert_seg_between(pquad,pseg_new,pquad->seglist_tail,NULL);
+    pseg_new = create_seg (thisseg_firstbyte, len);
+    insert_seg_between (pquad, pseg_new, pquad->seglist_tail, NULL);
 
-    return(rexlen);
+    return (rexlen);
 }
 
 
 
 /**********************************************************************/
 static segment *
-create_seg(
-    seqnum seq,
-    seglen len)
+create_seg (seqnum seq,
+	    seglen len)
 {
     segment *pseg;
 
-    pseg = (segment *)MallocZ(sizeof(segment));
+    pseg = (segment *) MallocZ (sizeof (segment));
 
     pseg->time = current_time;
     pseg->seq_firstbyte = seq;
     pseg->seq_lastbyte = seq + len - 1;
 
-    return(pseg);
+    return (pseg);
 }
 
 /**********************************************************************/
 static quadrant *
-create_quadrant(void)
+create_quadrant (void)
 {
     quadrant *pquad;
 
-    pquad = (quadrant *)MallocZ(sizeof(quadrant));
+    pquad = (quadrant *) MallocZ (sizeof (quadrant));
 
-    return(pquad);
+    return (pquad);
 }
 
 /********************************************************************/
 
 static quadrant *
-whichquad(
-    seqspace *sspace,
-    seqnum seq)
+whichquad (seqspace * sspace,
+	   seqnum seq)
 {
-    quadnum qid = QUADNUM(seq);
-    quadrant 	*pquad;
-    int		qix;
-    int		qix_next;
-    int		qix_opposite;
-    int		qix_prev;
+    quadnum qid = QUADNUM (seq);
+    quadrant *pquad;
+    int qix;
+    int qix_next;
+    int qix_opposite;
+    int qix_prev;
 
     /* optimize expected case, it's all set up correctly already */
     qix = qid - 1;
     if ((pquad = sspace->pquad[qix]) && pquad->next && pquad->prev)
-	return(pquad);
+	return (pquad);
 
     /* determine indices of "neighbor" quadrants */
-    qix_next     = (qix+1)%4;
-    qix_opposite = (qix+2)%4;
-    qix_prev     = (qix+3)%4;
+    qix_next = (qix + 1) % 4;
+    qix_opposite = (qix + 2) % 4;
+    qix_prev = (qix + 3) % 4;
 
     /* make sure that THIS quadrant exists */
-    if (sspace->pquad[qix] == NULL) {	
-	sspace->pquad[qix] = create_quadrant();
+    if (sspace->pquad[qix] == NULL) {
+	sspace->pquad[qix] = create_quadrant ();
     }
 
     /* make sure that the quadrant AFTER this one exists */
     if (sspace->pquad[qix_next] == NULL) {
-	sspace->pquad[qix_next] = create_quadrant();
+	sspace->pquad[qix_next] = create_quadrant ();
     }
 
     /* make sure that the quadrant BEFORE this one exists */
     if (sspace->pquad[qix_prev] == NULL) {
-	sspace->pquad[qix_prev] = create_quadrant();
+	sspace->pquad[qix_prev] = create_quadrant ();
     }
 
     /* clear out the opposite side, we don't need it anymore */
     if (sspace->pquad[qix_opposite] != NULL) {
-	freequad(&sspace->pquad[qix_opposite]);
+	freequad (&sspace->pquad[qix_opposite]);
 
 	sspace->pquad[qix_opposite] = NULL;
     }
@@ -317,15 +331,14 @@ whichquad(
     sspace->pquad[qix_next]->next = NULL;
     sspace->pquad[qix_prev]->prev = NULL;
 
-    return(sspace->pquad[qix]);
+    return (sspace->pquad[qix]);
 }
 
 
 
 /*********************************************************************/
 static void
-collapse_quad(
-    quadrant *pquad)
+collapse_quad (quadrant * pquad)
 {
     Bool freed;
     segment *pseg;
@@ -348,14 +361,14 @@ collapse_quad(
 	/* if this segment and the next one have both been ACKed and they */
 	/* "fit together", then collapse them into one (larger) segment   */
 	if (pseg->acked && pseg->next->acked &&
-	    (pseg->seq_lastbyte+1 == pseg->next->seq_firstbyte)) {
+	    (pseg->seq_lastbyte + 1 == pseg->next->seq_firstbyte)) {
 	    pseg->seq_lastbyte = pseg->next->seq_lastbyte;
 
 	    /* the new ACK count is the ACK count of the later segment */
 	    pseg->acked = pseg->next->acked;
 
 	    /* the new "transmit time" is the greater of the two */
-	    if (tv_gt(pseg->next->time,pseg->time))
+	    if (tv_gt (pseg->next->time, pseg->time))
 		pseg->time = pseg->next->time;
 
 	    tmpseg = pseg->next;
@@ -364,7 +377,7 @@ collapse_quad(
 		pseg->next->prev = pseg;
 	    if (tmpseg == pquad->seglist_tail)
 		pquad->seglist_tail = pseg;
-	    free(tmpseg);
+	    free (tmpseg);
 	    freed = TRUE;
 	}
 
@@ -382,11 +395,10 @@ collapse_quad(
 
 
 static void
-insert_seg_between(
-    quadrant *pquad,
-    segment *pseg_new,
-    segment *pseg_before,
-    segment *pseg_after)
+insert_seg_between (quadrant * pquad,
+		    segment * pseg_new,
+		    segment * pseg_before,
+		    segment * pseg_after)
 {
     /* fix forward pointers */
     pseg_new->next = pseg_after;
@@ -409,18 +421,17 @@ insert_seg_between(
 
 
 static enum t_ack
-rtt_ackin(
-    tcb *ptcb,
-    segment *pseg,
-    Bool rexmit_prev)
+rtt_ackin (tcb * ptcb,
+	   segment * pseg,
+	   Bool rexmit_prev)
 {
     double etime_rtt;
     enum t_ack ret;
 
-	u_long current_size=0;
+    u_long current_size = 0;
 
     /* how long did it take */
-    etime_rtt = elapsed(pseg->time,current_time);
+    etime_rtt = elapsed (pseg->time, current_time);
 
     if (rexmit_prev) {
 	/* first, check for the situation in which the segment being ACKed */
@@ -429,7 +440,7 @@ rtt_ackin(
 	ptcb->rtt_last = 0.0;	/* don't use this sample, it's very long */
 	etime_rtt = 0.0;
 
-	++ptcb->rtt_nosample;  /* no sample, even though not ambig */
+	++ptcb->rtt_nosample;	/* no sample, even though not ambig */
 	ret = NOSAMP;
     } else if (pseg->retrans == 0) {
 	ptcb->rtt_last = etime_rtt;
@@ -448,36 +459,35 @@ rtt_ackin(
 	/* Calculate the current_size of the segment,
 	   taking care of possible sequence space wrap around */
 
-	if ( pseg->seq_lastbyte > pseg->seq_firstbyte )
-	  current_size = pseg->seq_lastbyte - pseg->seq_firstbyte + 1;
+	if (pseg->seq_lastbyte > pseg->seq_firstbyte)
+	    current_size = pseg->seq_lastbyte - pseg->seq_firstbyte + 1;
 	else
-	   /* MAX_32 is 0x1,0000,0000
-	   So we don't need the "+ 1" while calculating the size here */
-	  current_size = (MAX_32 - pseg->seq_firstbyte)+pseg->seq_lastbyte;
+	    /* MAX_32 is 0x1,0000,0000
+	       So we don't need the "+ 1" while calculating the size here */
+	    current_size =
+		(MAX_32 - pseg->seq_firstbyte) + pseg->seq_lastbyte;
 
-	if ( !ptcb->rtt_full_size || (ptcb->rtt_full_size < current_size) )
-	{
-		/* Found a bigger segment.. Reset all stats. */
-		ptcb->rtt_full_size=current_size;
+	if (!ptcb->rtt_full_size || (ptcb->rtt_full_size < current_size)) {
+	    /* Found a bigger segment.. Reset all stats. */
+	    ptcb->rtt_full_size = current_size;
 
-		ptcb->rtt_full_min=etime_rtt;
-		ptcb->rtt_full_max=etime_rtt;
-		ptcb->rtt_full_sum=etime_rtt;
-		ptcb->rtt_full_sum2=(etime_rtt * etime_rtt);
-		ptcb->rtt_full_count=1;
-	}
-	else if (ptcb->rtt_full_size == current_size)
-	{
-		++ptcb->rtt_full_count;
+	    ptcb->rtt_full_min = etime_rtt;
+	    ptcb->rtt_full_max = etime_rtt;
+	    ptcb->rtt_full_sum = etime_rtt;
+	    ptcb->rtt_full_sum2 = (etime_rtt * etime_rtt);
+	    ptcb->rtt_full_count = 1;
+	} else if (ptcb->rtt_full_size == current_size) {
+	    ++ptcb->rtt_full_count;
 
-		if ((ptcb->rtt_full_min==0) || (ptcb->rtt_full_min>etime_rtt) )
-			ptcb->rtt_full_min = etime_rtt;
+	    if ((ptcb->rtt_full_min == 0)
+		|| (ptcb->rtt_full_min > etime_rtt))
+		ptcb->rtt_full_min = etime_rtt;
 
-		if (ptcb->rtt_full_max < etime_rtt)
-			ptcb->rtt_full_max = etime_rtt;
+	    if (ptcb->rtt_full_max < etime_rtt)
+		ptcb->rtt_full_max = etime_rtt;
 
-		ptcb->rtt_full_sum += etime_rtt;
-		ptcb->rtt_full_sum2 += (etime_rtt * etime_rtt);
+	    ptcb->rtt_full_sum += etime_rtt;
+	    ptcb->rtt_full_sum2 += (etime_rtt * etime_rtt);
 	}
 	ret = NORMAL;
     } else {
@@ -492,7 +502,7 @@ rtt_ackin(
 	ptcb->rtt_sum2_last += etime_rtt * etime_rtt;
 	++ptcb->rtt_count_last;
 
-	++ptcb->rtt_amback;  /* ambiguous ACK */
+	++ptcb->rtt_amback;	/* ambiguous ACK */
 
 	/* numbers not useful for plotting/dumping */
 	ptcb->rtt_last = 0.0;
@@ -503,30 +513,29 @@ rtt_ackin(
 
     /* dump RTT samples, if asked */
     if (dump_rtt && (etime_rtt != 0.0)) {
-	dump_rtt_sample(ptcb,pseg,etime_rtt);
+	dump_rtt_sample (ptcb, pseg, etime_rtt);
     }
 
     /* plot RTT samples, if asked */
     if (graph_rtt && (pseg->retrans == 0)) {
-	graph_rtt_sample(ptcb,pseg,etime_rtt);
+	graph_rtt_sample (ptcb, pseg, etime_rtt);
     }
 
-    return(ret);
+    return (ret);
 }
 
 
 
 static void
-rtt_retrans(
-    tcb *ptcb,
-    segment *pseg)
+rtt_retrans (tcb * ptcb,
+	     segment * pseg)
 {
     double etime;
 
     if (!pseg->acked) {
 	/* if it was acked, then it's been collapsed and these */
 	/* are no longer meaningful */
-	etime = elapsed(pseg->time,current_time);
+	etime = elapsed (pseg->time, current_time);
 	if (pseg->retrans > ptcb->retr_max)
 	    ptcb->retr_max = pseg->retrans;
 
@@ -536,7 +545,7 @@ rtt_retrans(
 	    ptcb->retr_min_tm = etime;
 
 	ptcb->retr_tm_sum += etime;
-	ptcb->retr_tm_sum2 += etime*etime;
+	ptcb->retr_tm_sum2 += etime * etime;
 	++ptcb->retr_tm_count;
     }
 
@@ -545,21 +554,31 @@ rtt_retrans(
 
 
 enum t_ack
-ack_in(
-    tcb *ptcb,
-    seqnum ack,
-    unsigned tcp_data_length)
+ack_in (tcb * ptcb,
+	seqnum ack,
+	unsigned tcp_data_length,
+	u_long eff_win)
 {
     quadrant *pquad;
     quadrant *pquad_prev;
     segment *pseg;
     Bool changed_one = FALSE;
     Bool intervening_xmits = FALSE;
-    timeval last_xmit = {0,0};
+    timeval last_xmit = { 0, 0 };
     enum t_ack ret = 0;
 
+    enum dup_ack_handling { BSD_VERSION = 1, /* Handling of duplicate ack's
+						based on the specifications of
+						BSD code */
+	LEGACY_VERSION = 2  /* Handling of duplicate ack's according to the 
+			       old versions of "tcptrace" */
+    };
+    enum dup_ack_handling dup_ack_type;	/* default type is the code based on
+					   BSD specifications */
+
+
     /* check each segment in the segment list for the PREVIOUS quadrant */
-    pquad = whichquad(ptcb->ss,ack);
+    pquad = whichquad (ptcb->ss, ack);
     pquad_prev = pquad->prev;
     for (pseg = pquad_prev->seglist_head; pseg != NULL; pseg = pseg->next) {
 	if (!pseg->acked) {
@@ -568,12 +587,12 @@ ack_in(
 	    ++ptcb->rtt_cumack;
 
 	    /* keep track of the newest transmission */
-	    if (tv_gt(pseg->time,last_xmit))
+	    if (tv_gt (pseg->time, last_xmit))
 		last_xmit = pseg->time;
 	}
     }
     if (changed_one)
-	collapse_quad(pquad_prev);
+	collapse_quad (pquad_prev);
 
     /* check each segment in the segment list for the CURRENT quadrant */
     changed_one = FALSE;
@@ -584,24 +603,58 @@ ack_in(
 	}
 
 	/* keep track of the newest transmission */
-	if (tv_gt(pseg->time,last_xmit))
+	if (tv_gt (pseg->time, last_xmit))
 	    last_xmit = pseg->time;
 
 	/* (ELSE) ACK covers this sequence */
 	if (pseg->acked) {
-	    /* already acked this one */
-	    ++pseg->acked;
-	    if (ack == (pseg->seq_lastbyte+1)) {
-		++ptcb->rtt_dupack; /* one more duplicate ack */
-		ret = CUMUL;
-		if (pseg->acked == 4) {
-		    /* some people say these CAN'T have data */
-		    if ((tcp_data_length == 0) ||
-			triple_dupack_allows_data) {
+	    /* default will be the BSD version, it can be changed by giving 
+	       '--turn_off_BSD_dupack' switch */
+	    dup_ack_type = (dup_ack_handling) ? BSD_VERSION : LEGACY_VERSION;
+
+	    /* default type is the specifications based on BSD code */
+	    switch (dup_ack_type) {
+	    case LEGACY_VERSION:
+		if (ack == (pseg->seq_lastbyte + 1)) {
+		    ++pseg->acked;	/* already acked this one */
+		    ++ptcb->rtt_dupack;	/* one more duplicate ack */
+		    ret = CUMUL;
+		    if (pseg->acked == 4) {
+			/* some people say these CAN'T have data */
+			if ((tcp_data_length == 0)
+			    || triple_dupack_allows_data) {
+			    ++ptcb->rtt_triple_dupack;
+			    ret = TRIPLE;
+			}
+		    }
+		}
+		break;
+	    case BSD_VERSION:
+		/* For an acknowledgement to be considered as duplicate ACK in 
+		   BSD version, following rules must be followed:
+		   1) the received segment should contain the biggest ACK TCP 
+		   has seen,
+		   2) the length of the segment containing dup ack should be 0,
+		   3) advertised window in this segment should not change,
+		   4) and there must be some outstanding data */
+
+		if ((ack == (pseg->seq_lastbyte + 1)) &&
+		      (ack == ptcb->ptwin->ack) &&
+		      (tcp_data_length == 0) &&
+		      (eff_win == ptcb->ptwin->win_last) &&
+		      (ptcb->owin_tot > 0)) {	
+		    ++ptcb->rtt_dupack;
+		    ret = CUMUL;
+
+		    /* already acked this one */
+		    ++pseg->acked;
+		    if (pseg->acked == 4) {
 			++ptcb->rtt_triple_dupack;
 			ret = TRIPLE;
 		    }
-		}
+		} else
+		    pseg->acked = 1;	/* received segment is not pure
+					   duplicate acknowledgement */
 	    }
 	    continue;
 	}
@@ -610,12 +663,12 @@ ack_in(
 	++pseg->acked;
 	changed_one = TRUE;
 
-	if (ack == (pseg->seq_lastbyte+1)) {
+	if (ack == (pseg->seq_lastbyte + 1)) {
 	    /* if ANY preceding segment was xmitted after this one,
 	       the the RTT sample is invalid */
-	    intervening_xmits = (tv_gt(last_xmit,pseg->time));
+	    intervening_xmits = (tv_gt (last_xmit, pseg->time));
 
-	    ret = rtt_ackin(ptcb,pseg,intervening_xmits);
+	    ret = rtt_ackin (ptcb, pseg, intervening_xmits);
 	} else {
 	    /* cumulatively ACKed */
 	    ++ptcb->rtt_cumack;
@@ -623,14 +676,13 @@ ack_in(
 	}
     }
     if (changed_one)
-	collapse_quad(pquad);
-    return(ret);
+	collapse_quad (pquad);
+    return (ret);
 }
 
 
 void
-freequad(
-    quadrant **ppquad)
+freequad (quadrant ** ppquad)
 {
     segment *pseg;
     segment *pseg_next;
@@ -638,25 +690,24 @@ freequad(
     pseg = (*ppquad)->seglist_head;
     while (pseg && pseg->next) {
 	pseg_next = pseg->next;
-	free(pseg);
+	free (pseg);
 	pseg = pseg_next;
     }
     if (pseg)
-	free(pseg);
-    free(*ppquad);
+	free (pseg);
+    free (*ppquad);
     *ppquad = NULL;
 }
 
 
 /* dump RTT samples in milliseconds */
 static void
-dump_rtt_sample(
-    tcb *ptcb,
-    segment *pseg,
-    double etime_rtt)
+dump_rtt_sample (tcb * ptcb,
+		 segment * pseg,
+		 double etime_rtt)
 {
     /* if the FILE is "-1", couldn't open file */
-    if (ptcb->rtt_dump_file == (MFILE *) -1) {
+    if (ptcb->rtt_dump_file == (MFILE *) - 1) {
 	return;
     }
 
@@ -665,64 +716,76 @@ dump_rtt_sample(
 	MFILE *f;
 	static char filename[15];
 
-	snprintf(filename,sizeof(filename),"%s2%s%s",
-		ptcb->host_letter, ptcb->ptwin->host_letter,
-		RTT_DUMP_FILE_EXTENSION);
+	snprintf (filename, sizeof (filename), "%s2%s%s",
+		  ptcb->host_letter, ptcb->ptwin->host_letter,
+		  RTT_DUMP_FILE_EXTENSION);
 
-	if ((f = Mfopen(filename,"w")) == NULL) {
-	    perror(filename);
-	    ptcb->rtt_dump_file = (MFILE *) -1;
+	if ((f = Mfopen (filename, "w")) == NULL) {
+	    perror (filename);
+	    ptcb->rtt_dump_file = (MFILE *) - 1;
 	}
 
 	if (debug)
-	    fprintf(stderr,"RTT Sample file is '%s'\n", filename);
+	    fprintf (stderr, "RTT Sample file is '%s'\n", filename);
 
 	ptcb->rtt_dump_file = f;
     }
 
-    Mfprintf(ptcb->rtt_dump_file,"%lu %lu\n",
-	    pseg->seq_firstbyte,
-	    (int)(etime_rtt/1000)  /* convert from us to ms */ );
+    Mfprintf (ptcb->rtt_dump_file, "%lu %lu\n",
+	      pseg->seq_firstbyte,
+	      (int) (etime_rtt / 1000) /* convert from us to ms */ );
 }
 
 
 
 /* graph RTT samples in milliseconds */
 static void
-graph_rtt_sample(
-    tcb *ptcb,
-    segment *pseg,
-    unsigned long etime_rtt)
+graph_rtt_sample (tcb * ptcb,
+		  segment * pseg,
+		  unsigned long etime_rtt)
 {
     char title[210];
 
     /* if the FILE is NULL, open file */
     if (ptcb->rtt_plotter == NO_PLOTTER) {
 	char *name_from, *name_to;
-	if (ptcb==&ptcb->ptp->a2b) {
+	if (ptcb == &ptcb->ptp->a2b) {
 	    name_from = ptcb->ptp->a_endpoint;
-	    name_to   = ptcb->ptp->b_endpoint;
+	    name_to = ptcb->ptp->b_endpoint;
 	} else {
 	    name_from = ptcb->ptp->b_endpoint;
-	    name_to   = ptcb->ptp->a_endpoint;
+	    name_to = ptcb->ptp->a_endpoint;
 	}
-	snprintf(title,sizeof(title),"%s_==>_%s (rtt samples)", name_from, name_to);
-	ptcb->rtt_plotter = new_plotter(ptcb,NULL,title,
-					"time","rtt (ms)",
-					RTT_GRAPH_FILE_EXTENSION);
-	plotter_perm_color(ptcb->rtt_plotter,"red");
+	snprintf (title, sizeof (title), "%s_==>_%s (rtt samples)",
+		  name_from, name_to);
+	ptcb->rtt_plotter =
+	    new_plotter (ptcb, NULL, title, "time", "rtt (ms)",
+			 RTT_GRAPH_FILE_EXTENSION);
+	plotter_perm_color (ptcb->rtt_plotter, "red");
 
 	if (graph_time_zero) {
 	    /* set graph zero points */
-	    plotter_nothing(ptcb->rtt_plotter, current_time);
+	    plotter_nothing (ptcb->rtt_plotter, current_time);
 	}
-	ptcb->rtt_line =
-	    new_line(ptcb->rtt_plotter, "rtt", "red");
+	ptcb->rtt_line = new_line (ptcb->rtt_plotter, "rtt", "red");
     }
 
     if (etime_rtt <= 1)
 	return;
 
-    extend_line(ptcb->rtt_line,
-		current_time, (int) (etime_rtt / 1000));
+    extend_line (ptcb->rtt_line, current_time, (int) (etime_rtt / 1000));
+}
+
+Bool IsRTO(tcb *ptcb, seqnum s) {
+  quadrant *pquad = whichquad(ptcb->ss,s);
+  segment *pseg;
+
+  for (pseg = pquad->seglist_head; pseg != NULL; pseg = pseg->next) {
+    if (s == (pseg->seq_lastbyte+1)) {
+      if (pseg->acked < 4) return TRUE;
+      else return FALSE;
+    }
+  }
+
+  return TRUE;
 }
