@@ -67,7 +67,7 @@
 #endif
 
 static char const GCC_UNUSED rcsid_tcptrace[] =
-    "@(#)$Header: /usr/local/cvs/tcptrace/tcptrace.h,v 5.75 2003/11/19 20:13:35 ryellapr Exp $";
+    "@(#)$Header: /usr/local/cvs/tcptrace/tcptrace.h,v 5.78 2004/11/04 21:55:38 sdo Exp $";
 
 
 #include <stdio.h>
@@ -263,17 +263,6 @@ extern timeval last_packet;
 extern u_long tcp_trace_count;
 extern u_long udp_trace_count;
 
-
-#ifdef OLD
-/* test 2 IP addresses for equality */
-#define IP_SAMEADDR(addr1,addr2) (((addr1).s_addr) == ((addr2).s_addr))
-/* test for an IP address lower than the second IP address */
-#define IP_LOWADDR(addr1,addr2) (((addr1).s_addr) < ((addr2).s_addr))
-
-/* copy IP addresses */
-#define IP_COPYADDR(toaddr,fromaddr) ((toaddr).s_addr = (fromaddr).s_addr)
-#endif
-
 typedef struct segment {
     seqnum	seq_firstbyte;	/* seqnumber of first byte */
     seqnum 	seq_lastbyte;	/* seqnumber of last byte */
@@ -874,9 +863,6 @@ void CompFormats(void);
 int CompIsCompressed(void);
 Bool FileIsStdin(char *filename);
 struct tcb *ptp2ptcb(tcp_pair *ptp, struct ip *pip, struct tcphdr *ptcp);
-void IP_COPYADDR (ipaddr *toaddr, ipaddr fromaddr);
-int IP_SAMEADDR (ipaddr addr1, ipaddr addr2);
-int IP_LOWADDR (ipaddr addr1, ipaddr addr2);
 void PcapSavePacket(char *filename, struct ip *pip, void *plast);
 void StringToArgv(char *buf, int *pargc, char ***pargv);
 void CopyAddr(tcp_pair_addrblock *, struct ip *pip,portnum,portnum);
@@ -909,9 +895,7 @@ void freequad(quadrant **);
 /* AVL tree support routines */
 enum AVLRES SnapInsert(ptp_snap **n, ptp_snap *new_node);
 enum AVLRES SnapRemove(ptp_snap **n, tcp_pair_addrblock address);
-int AVL_CheckHash(tcp_pair_addrblock *ptpa1,
-		  tcp_pair_addrblock *ptpa2, int *pdir);
-int AVL_CheckDir(tcp_pair_addrblock *ptpa1, tcp_pair_addrblock *ptpa2);
+int AVL_WhichDir(tcp_pair_addrblock *ptpa1, tcp_pair_addrblock *ptpa2);
     
 /* high-level line drawing */
 PLINE new_line(PLOTTER pl, char *label, char *color);
@@ -956,8 +940,12 @@ of bits as specified in RFC 2481 and draft-ietf-tsvwg-ecn-04.txt */
 #define B2A -1
 
 /* If the AVL node is to left or right in the AVL tree */
-#define LOW 2
-#define HIGH 3
+/* Words "LEFT" and "RIGHT" have already been taken in an enum
+ * above. Let us call them LT, RT just not to make it ambiguous for ourselves
+ * or the compiler or both :-)
+ */
+#define LT -2
+#define RT 2
 
 /*macros for maintaining the seqspace used for rexmit*/
 #define QUADSIZE	(0x40000000)
@@ -1168,7 +1156,7 @@ int snprintf_vms(char *str, size_t len, const char *fmt, ...);
 #define ADDR_ISV4(paddr) (ADDR_VERSION((paddr)) == 4)
 #define ADDR_ISV6(paddr) (ADDR_VERSION((paddr)) == 6)
 struct ipaddr *IPV4ADDR2ADDR(struct in_addr *addr4);    
-struct ipaddr *IPV6ADDR2ADDR(struct in6_addr *addr6);    
+struct ipaddr *IPV6ADDR2ADDR(struct in6_addr *addr6);
 
 /*
  * Macros to check for congestion experienced bits
